#!/bin/env python

import MySQLdb
import json
from StringIO import StringIO
from operator import itemgetter
import sys

db = sys.argv[1]

from ConfigParser import ConfigParser
parser = ConfigParser()
parser.read([sys.argv[2]])
if not parser.has_section('main'):
    raise KeyError("[main] section missing")

spec = {}
for sec in parser.sections():
    if sec == "main":
        if not parser.has_option('main','query'):
            raise KeyError("[main] section missing query parameter")
	spec['query'] = parser.get('main','query')
    else:
        if not parser.has_option(sec,'query'):
            raise KeyError("[%s] section missing query parameter"%(sec,))
        if not parser.has_option(sec,'params'):
            raise KeyError("[%s] section missing params parameter"%(sec,))
	spec[sec] = dict(query=parser.get(sec,'query'),
			 params=parser.get(sec,'params').split())

spec1 = {
  'query': """
select 
  film.film_id as film_id, 
  film.title as title,
  group_concat(DISTINCT category.name) as "jsonlist:categories",
  "jsonselect:actors" as actors
from film 
  join film_category
     on film.film_id = film_category.film_id
  join category
     on category.category_id = film_category.category_id
group by 
     film.film_id
""",
  'actors': {
     'params': [
        'film_id'
     ],
     'query': """
select
  actor.actor_id as actor_id,
  actor.first_name as first_name,
  actor.last_name as last_name
from
  actor
  join film_actor 
     on film_actor.actor_id = actor.actor_id
  join film
     on film.film_id = film_actor.film_id
where
  film.film_id = %s
"""
  }
}

spec1 = {
  'query': """
select 
  country.country_id as country_id,
  country.country as country,
  "jsonselect:city" as cities
from
  country
""",
  'city': {
    'params': [
      'country_id'
    ],
    'query': """
select
  city.city_id as city_id,
  city.city as city,
  "jsonselect:address" as addresses
from
  city
where
  city.country_id = %s
    """
  },
  'address': {
    'params': [
      'city_id'
    ],
    'query': """
select 
  address.address_id as _id,
  address.address as address,
  address.district as district,
  address.postal_code as postal_code
from
  address
where
  address.city_id = %s
    """
  }
}

db = MySQLdb.connect(host="127.0.0.1",user="root",db=db,port=3306)

def execute(db,spec,query=None,params=()):
  cursor = db.cursor()
  if not query:
    query = spec['query']
  cursor.execute(query,params)
  rows = []; headers = None
  for r in cursor.fetchall():
    if not headers:
      headers = map(str,map(itemgetter(0),cursor.description))
    row = dict(zip(headers,r))
    for k,v in list(row.items()):
      if isinstance(v,basestring) and v.startswith('jsonlist:'):
	key = v.split(':',1)[1]
	params = tuple(map(row.get,spec[key]['params']))
	subquery = spec[key]['query']
	v = execute(db,spec,subquery,params)
	row[k] = map(lambda d: d.values()[0],v)
      if isinstance(v,basestring) and v.startswith('jsonselect:'):
	key = v.split(':',1)[1]
	params = tuple(map(row.get,spec[key]['params']))
	subquery = spec[key]['query']
	v = execute(db,spec,subquery,params)
	row[k] = v
    rows.append(row)
  return rows

rows = execute(db,spec)
print json.dumps(dict(docs=rows),indent=2,sort_keys=True,skipkeys=True)

